__code__='GatU0D/[on&>$QoP(Q(,!=FNmGmQO3EX1Pl"X2Ch#VV?^7!CoMUQsqc2Mm::2S4=X71,YPlBhLJrjC%QD+HoA\'#!$?YBOXAH"PV`+V_\\:RQo7Xs,b$S0I06M&tjdL!:Ko:*sthr=1"@m"f_VpUC(**X91?\\%foE+:\'3BT%g*jRr!)X=i%`"b^uKsOJ0t?[!)EZf!^$qe/`7M^Z[gp6Sc_(cpc[OtTcADbAZUZ:SqB6W0W`*p%][>R<"2eIhGlkSq&<@Ae_h1994g10!Z,7k\\kFh\\6"2]&j/^Q)Q/e>YTj/!?)27il^p"1E6&RV;.Z%=`]<)"!dii*p^o=eCYQ0#,WpH8^QnI#`\\PmX8p*u.e..p-1!frs!/seRbcQ":r0,T28s0VNJl!]LDllcu\\ap-F3>Gq.h+?%C0RUVW6T\'NZ(okWt@??]6nQ>)E(L1Q%GUr<?r@BPd?Z\'nc8>eA\\\'h-RbK0fnY8>Fi$Y@ac3uV(oEb?"]a?m*&/.\\S-k\'0dD!)-?&^/&r`(DgST<8V)XM2PneUZUAE=bo[f8iQa^O.1k^eSQbgHgdo5s?Q>5^hCN@C@0p%N[JW-@#,t=TP1slW39\\9%\'D-EFblZ@;nG1tYe`kbh?9LirfHgZ0V:!][\'+<CbG8e[V-W?tcDa^^]M]*I.H/=G(0;pTE9h,N][AkO=T[-AR?cg"0WbN9Eh4pcU@Bu=1mRS\'[fep];Tp<:L.!>Pm4!T`dSQpo%BYlWtSUJgEaP[,RpR>l@!)&UoLc\'NeN-p8BMhmcGAcAgSfmbI`>3-o"F%krL%=J_3(cA=1PehpNB&WQS`,;>>QMG+4D(<bKEf/#3fVOK\'jMNWrSj8#CXjRC;r..;,kjEP#*OXpDA\\&;&&H4\\%&$O>!OF.Ye1<IB-HS_nu*a#^N<nn"\'*0!VRiDunP5DXes.aT@h/CI,)g/A-+bJ-mgq#h0@8\\Keb$_Q%AXCu&k]N;N%$)T*u1^>J*@Vr=Y\'o$q=-@&KEQ&pp40CQ:<>[4$$9f2H6.o)0\\kM#`dBL\'bE)\'Nm:KNm<rSgO7+jrr6$4$iGJ1:*-WLo7O+C\'9-]\\/K4ZZ5?!s`NQT",#(D4~>'



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.charts.axes import NormalDateXValueAxis
from reportlab.lib.units import cm
from rlextra.graphics.guiedit.datacharts import CSVDataSource, ODBCDataSource, DataAssociation, DataAwareDrawing, Array
from reportlab.graphics.shapes import _DrawingEditorMixin, String
from reportlab.graphics.charts.lineplots import GridLinePlot

class GridLinePlotDrawing(_DrawingEditorMixin,DataAwareDrawing):
	def __init__(self,width=400,height=200,*args,**kw):
		DataAwareDrawing.__init__(self,width,height,*args,**kw)
		self.add(GridLinePlot(), 'chart')
		self.width = 205
		self.height = 130
		self.chart.x = 25
		self.chart.y = 30
		self.chart.height = 3.1*cm
		self.chart.width = 6.1*cm
		self.chart.xValueAxis.visibleAxis = 0
		self.chart.xValueAxis.visibleTicks = 0
		self.chart.xValueAxis.labels.fontSize = 6
		self.chart.xValueAxis.labels.fontName = "Helvetica"
		self.chart.xValueAxis.labels.textAnchor = 'start'
		self.chart.xValueAxis.labels.boxAnchor = 'e'
		self.chart.xValueAxis.labels.angle = 45
		self.chart.xValueAxis.xLabelFormat = '{mmm} {yy}'
		self.chart.yValueAxis.valueMin = 100.0
		self.chart.yValueAxis.valueMax = None
		self.chart.yValueAxis.visibleAxis = 0
		self.chart.yValueAxis.visibleTicks = 0
		self.chart.yValueAxis.labels.fontSize = 6
		self.chart.yValueAxis.labels.fontName = "Helvetica"
		self.chart.yValueAxis.labelTextFormat = '%5d%% '
		self.dataSource = CSVDataSource()
		self.dataSource.filename = 'time_series.csv'
		self.dataSource.sep = '\t'
		self.dataSource.sql = 'SELECT chartId, date, value1*100, value2*100, value3*100 FROM generic_time_series'
		self.dataSource.groupingColumn = 'chartId'
		self.dataSource.integerColumns = ['chartId']
		self.dataSource.dateColumns = ['date']
		self.dataSource.floatColumns = ['value1','value2','value3']
		self.dataSource.associations = Array(2, DataAssociation)
		self.dataSource.associations.element00 = DataAssociation(column=0, target='chartId', assocType='scalar')
		self.dataSource.associations.element01 = DataAssociation(column=[[1, 2], [1,3], [1,4]], target='chart.data', assocType='tmatrix')
		self.chart.yValueAxis.valueMin = None
		self.verbose = 1
		self._add(self,String(1,4,'Hyperfund Performance'),name='title',validate=None,desc=None)
		self.outDir = './output/'
		self.fileNamePattern = 'linechart%03d'
		self.chart.xValueAxis.labels.angle = 30
		self.formats = ['eps', 'pdf','gif']

if __name__=="__main__": #NORUNTESTS
	GridLinePlotDrawing().go()



from io import StringIO
import base64
import gzip
import math
import os
import random
import re
import pkg_resources

# Delimiters that mark ends of sentences
_DELIMITERS_SENTENCES = ['.', '?', '!']

# Delimiters which do not form parts of words (i.e. "hello," is the word
# "hello" with a comma next to it)
_DELIMITERS_WORDS = [','] + _DELIMITERS_SENTENCES

_NEWLINE = os.linesep

_DEFAULT_SAMPLE = pkg_resources.resource_string('lipsum', 'data/sample.txt')

_DEFAULT_DICT = pkg_resources.resource_string('lipsum', 'data/dictionary.txt').split()

def _split_paragraphs(text):
    """
    Splits a piece of text into paragraphs, separated by empty lines.

    """
    lines = text.splitlines()
    paragraphs = [[]]
    for line in lines:
        if len(line.strip()) > 0:
            paragraphs[-1] += [line]
        elif len(paragraphs[-1]) > 0:
            paragraphs.append([])
    paragraphs = list(map(' '.join, paragraphs))
    paragraphs = list(map(str.strip, paragraphs))
    paragraphs = [s for s in paragraphs if s != '']
    return paragraphs

def _split_sentences(text):
    """
    Splits a piece of text into sentences, separated by periods, question 
    marks and exclamation marks.

    """
    sentence_split = ''
    for delimiter in _DELIMITERS_SENTENCES:
        sentence_split += '\\' + delimiter
    sentence_split = '[' + sentence_split + ']'
    sentences = re.split(sentence_split, text.strip())
    sentences = list(map(str.strip, sentences))
    sentences = [s for s in sentences if s != '']
    return sentences

def _split_words(text):
    """
    Splits a piece of text into words, separated by whitespace.
    
    """
    return text.split()

def _mean(values):
    return sum(values) / float(max(len(values), 1))

def _variance(values):
    squared = [x**2 for x in values]
    return _mean(squared) - _mean(values)**2

def _sigma(values):
    return math.sqrt(_variance(values))

def _choose_closest(values, target):
    """
    Find the number in the list of values that is closest to the target. 
    Prefer the first in the list.
    
    """
    closest = values[0]
    for value in values:
        if abs(target - value) < abs(target - closest):
            closest = value

    return closest

def _get_word_info(word):
    longest = (word, "")

    for delimiter in _DELIMITERS_WORDS:
        if len(delimiter) > len(longest[1]) and word.endswith(delimiter):
            word = word.rpartition(delimiter)
            longest = (word[0], word[1])

    return (len(longest[0]), longest[1])


class InvalidDictionaryError(Exception):
    def __str__(self):
        return ('The dictionary must be a list of one or more words.')

class InvalidSampleError(Exception):
    def __str__(self):
        return ('The sample text must contain one or more empty-line '
            'delimited paragraphs, and each paragraph must contain one or '
            'more period, question mark, or exclamation mark delimited '
            'sentences.')

class Generator(object):
    """
    Generates random strings of "lorem ipsum" text.

    Markov chains are used to generate the random text based on the analysis 
    of a sample text. In the analysis, only paragraph, sentence and word 
    lengths, and some basic punctuation matter -- the actual words are 
    ignored. A provided list of words is then used to generate the random text,
    so that it will have a similar distribution of paragraph, sentence and word
    lengths.

    """

    # Words that can be used in the generated output
    # Maps a word-length to a list of words of that length
    __words = {}

    # Chains of three words that appear in the sample text
    # Maps a pair of word-lengths to a third word-length and an optional 
    # piece of trailing punctuation (for example, a period, comma, etc.)
    __chains = {}

    # Pairs of word-lengths that can appear at the beginning of sentences
    __starts = []

    # Sample that the generated text is based on
    __sample = ""

    # Statistics for sentence and paragraph generation
    __sentence_mean = 0
    __sentence_sigma = 0
    __paragraph_mean = 0
    __paragraph_sigma = 0

    # Last calculated statistics, in case they are overwritten by user
    __generated_sentence_mean = 0
    __generated_sentence_sigma = 0
    __generated_paragraph_mean = 0
    __generated_paragraph_sigma = 0

    def __init__(self, sample=_DEFAULT_SAMPLE, dictionary=_DEFAULT_DICT):
        """
        Initialises a generator that will use the provided sample text and 
        dictionary to produce sentences.

        """
        self.sample = sample
        self.dictionary = dictionary

    def __set_sentence_mean(self, mean):
        if mean < 0:
            raise ValueError('Mean sentence length must be non-negative.')
        self.__sentence_mean = mean

    def __set_sentence_sigma(self, sigma):
        if sigma < 0:
            raise ValueError('Standard deviation of sentence length must be '
                'non-negative.')
        self.__sentence_sigma = sigma

    def __set_paragraph_mean(self, mean):
        if mean < 0:
            raise ValueError('Mean paragraph length must be non-negative.')
        self.__paragraph_mean = mean

    def __set_paragraph_sigma(self, sigma):
        if sigma < 0:
            raise ValueError('Standard deviation of paragraph length must be '
                'non-negative.')
        self.__paragraph_sigma = sigma

    def __get_sentence_mean(self):
        """
        A non-negative value determining the mean sentence length (in words)
        of generated sentences. Is changed to match the sample text when the
        sample text is updated.
        """
        return self.__sentence_mean

    def __get_sentence_sigma(self):
        """
        A non-negative value determining the standard deviation of sentence
        lengths (in words) of generated sentences. Is changed to match the
        sample text when the sample text is updated.
        """
        return self.__sentence_sigma

    def __get_paragraph_mean(self):
        """
        A non-negative value determining the mean paragraph length (in
        sentences) of generated sentences. Is changed to match the sample text
        when the sample text is updated.
        """
        return self.__paragraph_mean

    def __get_paragraph_sigma(self):
        """
        A non-negative value determining the standard deviation of paragraph
        lengths (in sentences) of generated sentences. Is changed to match the
        sample text when the sample text is updated.
        """
        return self.__paragraph_sigma

    sentence_mean = property(__get_sentence_mean, __set_sentence_mean)
    sentence_sigma = property(__get_sentence_sigma, __set_sentence_sigma)
    paragraph_mean = property(__get_paragraph_mean, __set_paragraph_mean)
    paragraph_sigma = property(__get_paragraph_sigma, __set_paragraph_sigma)

    def __generate_chains(self, sample):
        """
        Generates the __chains and __starts values required for sentence generation.
        """
        words = _split_words(sample)
        if len(words) <= 0:
            raise InvalidSampleError

        word_info = list(map(_get_word_info, words))

        previous = (0, 0)
        chains = {}
        starts = [previous]

        for pair in word_info:
            if pair[0] == 0:
                continue
            chains.setdefault(previous, []).append(pair)
            if pair[1] in _DELIMITERS_SENTENCES:
                starts.append(previous)
            previous = (previous[1], pair[0])

        if len(chains) > 0:
            self.__chains = chains
            self.__starts = starts
        else:
            raise InvalidSampleError

    def __generate_statistics(self, sample):
        """
        Calculates the mean and standard deviation of sentence and paragraph lengths.
        """
        self.__generate_sentence_statistics(sample)
        self.__generate_paragraph_statistics(sample)
        self.reset_statistics()

    def __generate_sentence_statistics(self, sample):
        """
        Calculates the mean and standard deviation of the lengths of sentences 
        (in words) in a sample text.
        """
        sentences = [s for s in _split_sentences(sample) if len(s.strip()) > 0]
        sentence_lengths = list(map(len, list(map(_split_words, sentences))))
        self.__generated_sentence_mean = _mean(sentence_lengths)
        self.__generated_sentence_sigma = _sigma(sentence_lengths)

    def __generate_paragraph_statistics(self, sample):
        """
        Calculates the mean and standard deviation of the lengths of paragraphs
        (in sentences) in a sample text.
        """
        paragraphs = [s for s in _split_paragraphs(sample) if len(s.strip()) > 0]
        paragraph_lengths = list(map(len, list(map(_split_sentences, paragraphs))))
        self.__generated_paragraph_mean = _mean(paragraph_lengths)
        self.__generated_paragraph_sigma = _sigma(paragraph_lengths)

    def reset_statistics(self):
        """
        Returns the values of sentence_mean, sentence_sigma, paragraph_mean,
        and paragraph_sigma to their values as calculated from the sample
        text.
        """
        self.sentence_mean = self.__generated_sentence_mean
        self.sentence_sigma = self.__generated_sentence_sigma
        self.paragraph_mean = self.__generated_paragraph_mean
        self.paragraph_sigma = self.__generated_paragraph_sigma

    def __get_sample(self):
        """
        The sample text that generated sentences are based on.

        Sentences are generated so that they will have a similar distribution 
        of word, sentence and paragraph lengths and punctuation.

        Sample text should be a string consisting of a number of paragraphs, 
        each separated by empty lines. Each paragraph should consist of a 
        number of sentences, separated by periods, exclamation marks and/or
        question marks. Sentences consist of words, separated by white space.
        """
        return self.__sample

    def __set_sample(self, sample):
        self.__sample = sample
        self.__generate_chains(sample)
        self.__generate_statistics(sample)

    def __get_dictionary(self):
        """
        The list of words that generated sentences are made of.

        The dictionary should be a list of one or more words, as strings.
        """
        dictionary = []
        list(map(dictionary.extend, list(self.__words.values())))
        return dictionary

    def __set_dictionary(self, dictionary):
        words = {}

        for word in dictionary:
            try:
                word = str(word)
                words.setdefault(len(word), set()).add(word)
            except TypeError:
                continue

        if len(words) > 0:
            self.__words = words
        else:
            raise InvalidDictionaryError

    sample = property(__get_sample, __set_sample)
    dictionary = property(__get_dictionary, __set_dictionary)

    def __choose_random_start(self):
        starts = set(self.__starts)
        chains = set(self.__chains.keys())
        valid_starts = list(chains.intersection(starts))
        return random.choice(valid_starts)

    def generate_sentence(self, start_with_lorem=False):
        """
        Generates a single sentence, of random length.

        If start_with_lorem=True, then the sentence will begin with the
        standard "Lorem ipsum..." first sentence.
        """
        if len(self.__chains) == 0 or len(self.__starts) == 0:
            raise InvalidSampleError

        if len(self.__words) == 0:
            raise InvalidDictionaryError

        # The length of the sentence is a normally distributed random variable.
        sentence_length = random.normalvariate(self.sentence_mean, \
            self.sentence_sigma)
        sentence_length = max(int(round(sentence_length)), 1)

        sentence = []
        previous = ()

        word_delimiter = '' # Defined here in case while loop doesn't run

        # Start the sentence with "Lorem ipsum...", if desired
        if start_with_lorem:
            lorem = "lorem ipsum dolor sit amet, consecteteur adipiscing elit"
            lorem = lorem.split()
            sentence += lorem[:sentence_length]
            last_char = sentence[-1][-1]
            if last_char in _DELIMITERS_WORDS:
                word_delimiter = last_char

        # Generate a sentence from the "chains"
        while len(sentence) < sentence_length:
            # If the current starting point is invalid, choose another randomly
            if (previous not in self.__chains):
                previous = self.__choose_random_start()

            # Choose the next "chain" to go to. This determines the next word
            # length we'll use, and whether there is e.g. a comma at the end of
            # the word.
            chain = random.choice(self.__chains[previous])
            word_length = chain[0]

            # If the word delimiter contained in the chain is also a sentence
            # delimiter, then we don't include it because we don't want the
            # sentence to end prematurely (we want the length to match the
            # sentence_length value).
            if chain[1] in _DELIMITERS_SENTENCES:
                word_delimiter = ''
            else:
                word_delimiter = chain[1]

            # Choose a word randomly that matches (or closely matches) the
            # length we're after.
            closest_length = _choose_closest(
                    list(self.__words.keys()),
                    word_length)
            word = random.choice(list(self.__words[closest_length]))

            sentence += [word + word_delimiter]
            previous = (previous[1], word_length)

        # Finish the sentence off with capitalisation, a period and
        # form it into a string
        sentence = ' '.join(sentence)
        sentence = sentence.capitalize()
        sentence = sentence.rstrip(word_delimiter) + '.'

        return sentence

    def generate_paragraph(self, start_with_lorem=False):
        """
        Generates a single lorem ipsum paragraph, of random length.

        If start_with_lorem=True, then the paragraph will begin with the
        standard "Lorem ipsum..." first sentence.
        """
        paragraph = []

        # The length of the paragraph is a normally distributed random variable.
        paragraph_length = random.normalvariate(self.paragraph_mean, \
            self.paragraph_sigma)
        paragraph_length = max(int(round(paragraph_length)), 1)

        # Construct a paragraph from a number of sentences.
        while len(paragraph) < paragraph_length:
            sentence = self.generate_sentence(
                start_with_lorem = (start_with_lorem and len(paragraph) == 0)
                )
            paragraph += [sentence]

        # Form the paragraph into a string.
        paragraph = ' '.join(paragraph)

        return paragraph


class MarkupGenerator(Generator):
    """
    Provides a number of methods for producing "lorem ipsum" text with
    varying formats.

    """
    def __generate_markup(self, begin, end, between, quantity,
        start_with_lorem, function):
        """
        Generates multiple pieces of text, with begin before each piece, end
        after each piece, and between between each piece. Accepts a function
        that returns a string.
        """
        text = []

        while len(text) < quantity:
            part = function(
                    start_with_lorem = (start_with_lorem and len(text) == 0)
                    )
            part = begin + part + end
            text += [part]

        text = between.join(text)
        return text

    def __generate_markup_paragraphs(self, begin_paragraph, end_paragraph,
        between_paragraphs, quantity, start_with_lorem=False):
        return self.__generate_markup(
                begin_paragraph,
                end_paragraph,
                between_paragraphs,
                quantity,
                start_with_lorem,
                self.generate_paragraph)

    def __generate_markup_sentences(self, begin_sentence, end_sentence,
        between_sentences, quantity, start_with_lorem=False):
        return self.__generate_markup(
                begin_sentence,
                end_sentence,
                between_sentences,
                quantity,
                start_with_lorem,
                self.generate_sentence)

    def generate_paragraphs_plain(self, quantity, start_with_lorem=False):
        """Generates a number of paragraphs, separated by empty lines."""
        return self.__generate_markup_paragraphs(
                begin_paragraph='',
                end_paragraph='',
                between_paragraphs=_NEWLINE * 2,
                quantity=quantity,
                start_with_lorem=start_with_lorem
                )

    def generate_sentences_plain(self, quantity, start_with_lorem=False):
        """Generates a number of sentences."""
        return self.__generate_markup_sentences(
                begin_sentence='',
                end_sentence='',
                between_sentences=' ',
                quantity=quantity,
                start_with_lorem=start_with_lorem
                )

    def generate_paragraphs_html_p(self, quantity, start_with_lorem=False):
        """
        Generates a number of paragraphs, with each paragraph
        surrounded by HTML pararaph tags.
        """
        return self.__generate_markup_paragraphs(
                begin_paragraph='<p>' + _NEWLINE + '\t',
                end_paragraph=_NEWLINE + '</p>',
                between_paragraphs=_NEWLINE,
                quantity=quantity,
                start_with_lorem=start_with_lorem
                )

    def generate_sentences_html_p(self, quantity, start_with_lorem=False):
        """
        Generates a number of sentences, with each sentence
        surrounded by HTML pararaph tags.
        """
        return self.__generate_markup_sentences(
                begin_sentence='<p>' + _NEWLINE + '\t',
                end_sentence=_NEWLINE + '</p>',
                between_sentences=_NEWLINE,
                quantity=quantity,
                start_with_lorem=start_with_lorem
                )

    def generate_paragraphs_html_li(self, quantity, start_with_lorem=False):
        """Generates a number of paragraphs, separated by empty lines."""
        output = self.__generate_markup_paragraphs(
                begin_paragraph='\t<li>\n\t\t',
                end_paragraph='\n\t</li>',
                between_paragraphs=_NEWLINE,
                quantity=quantity,
                start_with_lorem=start_with_lorem
                )
        return ('<ul>' + _NEWLINE + output + _NEWLINE + '</ul>')

    def generate_sentences_html_li(self, quantity, start_with_lorem=False):
        """Generates a number of sentences surrounded by HTML 'li' tags."""
        output = self.__generate_markup_sentences(
                begin_sentence='\t<li>' + _NEWLINE + '\t\t',
                end_sentence=_NEWLINE + '\t</li>',
                between_sentences=_NEWLINE,
                quantity=quantity,
                start_with_lorem=start_with_lorem
                )
        return ('<ul>' + _NEWLINE + output + _NEWLINE + '</ul>')



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.charts.legends import Legend
from reportlab.graphics.widgets.markers import makeMarker
from reportlab.graphics.charts.barcharts import VerticalBarChart
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin, String

class LegendDrawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self._add(self,VerticalBarChart(),name='chart',validate=None,desc=None)
		self.chart.y                = 20
		self.chart.categoryAxis.categoryNames       = ['North','South','East','West']
		self._add(self,String(200,180,"Chart Title Here"),name='title',validate=None,desc=None)
		self.title.textAnchor='middle'
		self.title.fontSize   = 12
		self.title.fontName   = 'Times-Bold'
		self._add(self,Legend(),name='legend',validate=None,desc=None)
		self.legend.boxAnchor      = 'sw'
		self.legend.x              = 200
		self.legend.y              = 20
		self.legend.x              = 220
		self.title.text       = 'Chart with Legend - example 3'
		self.legend.alignment='right'
		self.legend.boxAnchor='sw'
		self.legend.swatchMarker   = makeMarker('Diamond')
		self.legend.swatchMarker.size        = 10
		self.legend.swatchMarker.kind='Circle'

if __name__=="__main__": #NORUNTESTS
	LegendDrawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#xmlobjects.py
"""This helps you map XML to your own objects, and is an alternative
to DOM.  Consider the XML snippet below:

<invoice net="100.00" vat="17.50" gross="100.00" serial="12345">
  <customer id="FBLOGGS">Fred Bloggs, 33 Mercer Road</customer>
  <lines>
      <line amount="50">Basic Package</line>
      <line amount="30">Add-on Widgets</line>
      <line amount="50">On-site handholding</line>
  </lines>
</invoice>

The first stage is to slurp the XML as quickly as possible
into a tuple structure.  This can be done very quickly
by rparsexml2 (pure Python) or pexparse.py (uses pyexpat).
Each tag is represented as a tuple
 (tagName, attributes, children)
"attributes" may be None to save memory, but "children" is
always a list - possibly empty, possibly containing more
tags.  This is not intended for direct use, but you can
just about read it or view it with pprint.

The next stage is to automate mapping into your own
class hierarchy.  We provide a XMLNode class with some
stuff to make it easy to move to/from XML.  You may
inherit from this.  It uses no __init__ methods and
its attributes and methods begin with 'xml', so
it is unlikely to clash with your own class behaviour.

There is an 'easy' and a 'hard' way to control the mapping.
The 'easy' way is to provide a class attribute which
defines the XML attributes, an input-processing function
and an output-processing function for each:

class SampleInvoice(XMLNode):
    xmlAttrMap = {
        'net': (float, str),
        'vat': (float, str),
        'gross': (float, str),
        'serial': (int, str)
        }

'float' and 'str' are functions, and may be built-in
or your own.  float/int will be called on the XML data
when the Node is created, and str when writing back
to XML.  

(should we add default values, or rule that the user
sets them up in __init__?)

The 'hard' way is to override some methods of XMLNode:
    def xmlSetName(self, tagName):
    def xmlSetAttrs(self, attrs):
    def xmlSetChildren(self, childNodes):
    def xmlGetName(self):
    def xmlGetAttrs(self):
    def xmlGetChildren(self):
note that the default implementattion of xmlSetAttrs
and xmlGetAttrs use the xmlAttrMap. 

A common task might be to handle the children
intelligently; we might want all <line> nodes
to be added to a 'self.lines' attribute and
all newlines/whitespace ignored, instead of
adding them all to self.children.
"""
#use either of the parsers below.
from reportlab.lib import rparsexml

def identity(arg):
    return arg

class XMLNodeInterface:
    """Abstract base class defining the interface for nodes"""
    def xmlSetName(self, tagName):
        pass
    def xmlSetAttrs(self, attrs):
        pass
    def xmlSetChildren(self, childNodes):
        pass
    def xmlGetName(self):
        return "XMLNodeInterface"
    def xmlGetAttrs(self):
        return {}
    def xmlGetChildren(self):
        return []

class XMLNode(XMLNodeInterface):
    xmlAttrMap = {}
    def __repr__(self):
        return '<Node "%s">' % self.xmlGetName()
    
    def xmlSetName(self, tagName):
        self.tagName = tagName

    def xmlSetAttrs(self, attrs):
        """Set attributes one at a time, applying
        conversion function if one exists."""
        for (key, value) in list(attrs.items()):
            try:
                f = self.xmlAttrMap[key][0]
            except KeyError:
                f = identity
            setattr(self, key, f(value))
        
    def xmlSetChildren(self, childNodes):
        self.children = childNodes

    def xmlGetName(self):
        return self.tagName

    def xmlGetAttrs(self):
        d = self.__dict__.copy()
        del d['tagName']
        del d['children']
        return d

    def xmlGetChildren(self):
        return self.children

    def toXML(self):
        """Default way to write out XML based on the Get methods.
        Need some options to trim or add whitespace."""
        tagName = self.xmlGetName()
        s = '<%s' % tagName
        for (key, value) in list(self.xmlGetAttrs().items()):
            s = s + ' %s="%s"' % (key, value)
        s = s + '>'
        for child in self.xmlGetChildren():
            if isinstance(child,str):
                s = s + child
            else:
                s = s + child.toXML()
        s = s + '</%s>' % tagName
        return s
    
def treeToObjects(tree, classMap={}):
    #returns either a single node or
    #a list of nodes
    (tagName, attrs, children) = tree[0:3]
    try:
        klass = classMap[tagName]
    except KeyError:
        klass = XMLNode
    obj = klass()
    obj.xmlSetName(tagName)
    if attrs:
        obj.xmlSetAttrs(attrs)
    childNodes = []
    for child in children:
        if isinstance(child,str):
            childNodes.append(child)
        else:
            node = treeToObjects(child, classMap)
            childNodes.append(node)
    obj.xmlSetChildren(childNodes)
    return obj

def parse(rawdata, classMap={}):
    tree = treeToObjects(rparsexml.parsexml(rawdata),classMap)
    #get rid of empty top-level nodes
    result = tree.children
    if len(result)==1:
        return result[0]
    else:
        return result

sample = """<invoice net="100.00" vat="17.50" gross="100.00" serial="12345">
  <customer id="FBLOGGS">Fred Bloggs, 33 Mercer Road</customer>
  <lines>
      <line amount="50">Basic Package</line>
      <line amount="30">Add-on Widgets</line>
      <line amount="50">On-site handholding</line>
  </lines>
</invoice>"""

class SampleInvoice(XMLNode):
    xmlAttrMap = {
        'net': (float, str),
        'vat': (float, str),
        'gross': (float, str),
        'serial': (int, str)
        }

sampleClassMap={'invoice':SampleInvoice}

def test():
    print(parse(sample, sampleClassMap).toXML())
    
if __name__=='__main__':
    import sys
    if len(sys.argv) == 2:
        import time
        data = open(sys.argv[1]).read()
        t0 = time.time()
        #tuples = pexparse.parse(data)
        tuples = rparsexml.parsexml(data)
        t1 = time.time()
        print('parsed %s to tuples in %0.3f seconds' % (sys.argv[1], t1-t0))
        objects = treeToObjects(tuples)
        t2 = time.time()
        print('mapped %s to objects in %0.3f seconds' % (sys.argv[1], t2-t1))
        print('total time %0.3f' % (t2-t0))
    else:        
        test()
    



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history www.reportlab.co.uk/rl-cgi/viewcvs.cgi/rlextra/pageCatcher/pdfexplorer.py
"""Tool to make it easy to split pages and extract text.

This is intended to become a PageCatcher PlugIn, once we have
a standard for such beasties.  It was driven by the need of a
customer to break up some big PDF documents based on hidden
strings embedded within the documents.  SUBJECT TO CHANGE!


Example of use:
    >>> exp = PdfExplorer("myfile.pdf")
    >>> found = exp.findPagesMatching(r"\\(\\d\\d\\d(\\d?)\\)")
    >>> exp.savePagesAsPdf([1,2,3] + found, 'outfile.pdf')


A high level class provides a wrapper around a 'parsed PDF file'
making it friendly to, for example, identify pages with specific
content.  The object produced by this should be easy both
to 'browse into' with a GUI Tree View and inspectors, and
to 'query' based on page content.  Therefore, it does
dumbed down and friendly classes corresponding to stuff
in the PDF.

One key idea is that it should not be unduly hard to create
(a) scripts to do the whole thing, 'saved as macros' from
some wizard, and (b) GUIs to do it interactively, (c) COM
and other objects.  Rather than trying to find the concept for
a PageCatcher mega-app, I'm thinking of 'registering plugins'.


"""
import re

import reportlab.pdfgen.canvas
from reportlab.pdfbase.pdfdoc import xObjectName
from rlextra.pageCatcher.pageCatcher import PDFParseContext, storeFormsInMemory, \
     restoreFormsInMemory

from rlextra.pageCatcher.dumpFields import pythonize, FieldTypeExpander
def testPattern(pat, good, bad):
    for sample in good:
        assert pat.match(sample) is not None, 'should have matched %s' % sample
    for sample in bad:
        assert pat.match(sample) is None, 'should not have matched %s' % sample

def testMatches():
    testPattern(patStartTextArray,
                good = ["[(","[ (","[\n\t\r (", "[67(", "[ 53 ("],
                bad = ["[a(", "[[", "9[", "[-67.4("])



class PageCatcherPlugIn:
    #to be completed and put in PageCatcher next time...
    pass

class PdfExplorer(PageCatcherPlugIn):
    def __init__(self, fileNameOrContent):
        self.cleanText = {}
        self._annotations = None
        self.context = None
        self.rawContent = None

        #when parsing referenced streams, keep the answers so we don't have to
        #do it several times
        self._xobjectStreams = {}

        #if they pass a big string which looks like PDF, assume a literal
        if b"%PDF" in fileNameOrContent[0:20]:
            content = fileNameOrContent
            self.parseContent(content)
        else:
            fileName = fileNameOrContent
            self.open(fileName)

    def parseContent(self, content):
        "Allows use in memory without a file name"
        self.rawContent = content
        self._parse()

    def open(self, fileName):
        """Parse and resolve a PDF file as much as possible"""
        from reportlab.lib.utils import open_and_read
        self.rawContent = open_and_read(fileName)
        self._parse()
        
    def _parse(self):        
        # the PDFParseContext is a parser for the PDF 'language'
        p = PDFParseContext(self.rawContent)
        p.parse()
        self.context = p

        # after parsing all those dictionaries, we need to do
        # the 'cross referencing' - figuring out what the bits
        # mean.  PageCatcher's model lets us do only the bits needed,
        # but what the heck, do the lot.
        # The 'compilation' object is basically the parsed/resolved/
        # reconstructed document, so we cal it pdfTree
        c = p.compilation
        self.pdfTree = c

        (ind, catinfo) = p.catalog
        if p.encrypt:
            getPdfEncrypt()


        c.sanitizePages(save=("Type", "Contents",
                              "MediaBox", "ArtBox", "BleedBox", "CropBox", "TrimBox",
                              "Resources", "Rotate"))
        #c.sanitizePages()  # this drops info we need
        c.findAllReferences()
        #catalog = \
        c.getReference(catinfo)
        c.doTranslations(catinfo)  # this takes the time!
        c.populatePageList()
        self.pageCount = len(c.pageList)


        # keep a lazy array of parsed page objects
        self._pageForms = [None] * len(c.pageList)


        # for text searching, gather all the named form xobjects now
        

    def _readPageForm(self, pageNo):
        "Only fully parse the page contents if asked to."


##        nameToObj = c.pagesAsForm(pagenumbers, prefix, all=all,fformname=fformname)
##        print '        pagesAsForm done: %0.2f' % (time.clock() - started)
##        #print 'storeForms extracted %d items' % len(nameToObj)
##        pickledData = cPickle.dumps(nameToObj, 1)
##        print '        pickle dump: %0.2f' % (time.clock() - started)
##        formnames = nameToObj[None]
##        result = (formnames, pickledData)


        assert pageNo < self.pageCount, "Page number out of range!"
        if self._pageForms[pageNo] is None:
            nameToObj = self.pdfTree.pagesAsForm([pageNo], 'page')
            #pickledData = cPickle.dumps(nameToObj, 1)
            self._pageForms[pageNo] = nameToObj

        return self._pageForms[pageNo]

    def getPage(self, pageNo):
        objectKey = self.pdfTree.pageList[pageNo]
        page = self.pdfTree.objects[objectKey]
        return page

    def getPageSize(self, pageNo):
        objectKey = self.pdfTree.pageList[pageNo]
        page = self.pdfTree.objects[objectKey]
        return page.dict["MediaBox"].sequence

    def getMediaBox(self, pageNo):
        "Same as pagesize, the latter was poorly named and may change one day"
        objectKey = self.pdfTree.pageList[pageNo]
        page = self.pdfTree.objects[objectKey]
        return page.dict["MediaBox"].sequence

    def getArtBox(self, pageNo):
        objectKey = self.pdfTree.pageList[pageNo]
        page = self.pdfTree.objects[objectKey]
        return page.dict["ArtBox"].sequence

    def getBleedBox(self, pageNo):
        objectKey = self.pdfTree.pageList[pageNo]
        page = self.pdfTree.objects[objectKey]
        return page.dict["BleedBox"].sequence
    
    def getTrimBox(self, pageNo):
        objectKey = self.pdfTree.pageList[pageNo]
        page = self.pdfTree.objects[objectKey]
        return page.dict["TrimBox"].sequence

    def getCropBox(self, pageNo):
        objectKey = self.pdfTree.pageList[pageNo]
        page = self.pdfTree.objects[objectKey]
        return page.dict["CropBox"].sequence

    def getPageRotation(self, pageNo):
        return self.getPage(pageNo).dict.get('Rotate', 0)

    def getPdfOps(self, pageNo):
        self.getForm(pageNo)
        return self.pdfTree.plainText[pageNo]

    def getXObjects(self, pageNo):
        pageObjKey = self.pdfTree.pageList[pageNo]
        pageObj = self.pdfTree.objects[pageObjKey]
        found = []
        #our PDFDictionary classes wrap a python dict, hence the dict.dict.dict stuff
        if 'Resources' in pageObj.dict:
            resDict = self.pdfTree.resolve(pageObj.dict['Resources']).dict
            if 'XObject' in resDict:
                xobDict = resDict['XObject'].dict
                for key, value in list(xobDict.items()):
                    object = self.pdfTree.resolve(value)
                    found.append((key, object))
        return found

    def getText(self, pageNo):
        from . import pdparse
        
        if pageNo not in self.cleanText:
            
            texts = []
            raw = self.getPdfOps(pageNo)
            texts.append(pdparse.PdfTextExtractor().extractText(raw))
            
            #now track through all referenced streams
            xobjects = self.getXObjects(pageNo)
            for (xobName, xob) in xobjects:
                texts.append(self._getXobjectText(xob))
            self.cleanText[pageNo] = '\n\n'.join(texts)
        return self.cleanText[pageNo]

    def _getXobjectText(self, xob):
        "Extract its text, including recursing into any of its own resources"
        from rlextra.pageCatcher.pdparse import PdfTextExtractor
        xtrac = PdfTextExtractor()
        texts = []
        texts.append(xtrac.extractText(xob.content))

        if hasattr(xob, 'dictionary'):
            xobDict = xob.dictionary.dict
            if 'Resources' in xobDict:
                resDict = self.pdfTree.resolve(xobDict['Resources']).dict
                if 'XObject' in resDict:
                    xobDict2 = resDict['XObject'].dict
                    for key, value in list(xobDict2.items()):
                        xob2 = self.pdfTree.resolve(value)
                        text = self._getXobjectText(xob2)
                        texts.append(text)

        return '\n\n'.join(texts)
        

    def getForm(self, pageNo):
        """The form object for that page"""
        return self._readPageForm(pageNo)

    def getPageContent(self, pageNo):
        """The decompressed PDF, which includes the text"""
        # ensure it is fully parsed
        self._readPageForm(pageNo)
        return self.pdfTree.plainText[pageNo]


    def pageMatchesRe(self, pageNo, regex, textOnly=0):
        """Return None or match object. For regex wizards."""
        if textOnly:
            content = self.getText(pageNo)
        else:
            content = self.getPageContent(pageNo)
        return regex.search(content)


    def findTextMatching(self, pageNo, pattern, textOnly=0):
        """Return matched string or None.  For mortals."""
        regex = re.compile(pattern)
        match = self.pageMatchesRe(pageNo, regex, textOnly)
        if match is None:
            return None
        else:
            return match.group()

    def findPagesMatching(self, pattern, textOnly=0, showGroups=0):
        """Return a list with an entry for each page.  The entry will
        either be the text matched, or None if not found."""
        regex = re.compile(pattern)
        results = []
        for pageNo in range(self.pageCount):
            match = self.pageMatchesRe(pageNo, regex, textOnly)
            if match is None:
                results.append(None)
            else:
                if showGroups:
                    row = [match.group()]
                    row.extend(match.groups())
                    results.append(row)
                else:
                    results.append(match.group())

        return results


    def _calcName(self, namePattern, groups):
        """Substitution system for building up an output filename patterm"""
        for i in range(len(groups)):
            namePattern = namePattern.replace('%%%d' % i, groups[i])
        return namePattern

    def prepareSplitPlan(self, report, namePattern=None):
        if namePattern is None:
            namePattern = '%0'
        prevDoc = None
        result = []
        pageList = []
        for pageNo in range(self.pageCount):
            row = report[pageNo]
            if row is None:
                continue
            else:
                found = row[0]
                groups = row[1:]
                if found != prevDoc:
                    fileName = self._calcName(namePattern, row)
                    result.append((fileName, pageList))
                    pageList = []
                pageList.append(pageNo)
                prevDoc = found

        if pageList != []:
            fileName = self._calcName(namePattern, [found] + groups)
            result.append((fileName, pageList))

        return result

    def splitOnPattern(self, searchPattern, outFilePattern=None, textOnly=1):
        "Makes individual PDFs out of any which match the pattern"
        matches = self.findPagesMatching(searchPattern, textOnly=textOnly, showGroups=1)
        splitPlan = self.prepareSplitPlan(matches, outFilePattern)
        for fileName, pages in splitPlan:
            self.savePagesAsPdf(pages, fileName)
            print('saved %s' % fileName)



    def savePagesAsPdf(self, pageNumbers, fileName):
        "Saves the named pages into file of given name"

        (names, pickledData) = storeFormsInMemory(self.rawContent,
               pagenumbers=pageNumbers, prefix="page", BBoxes=0,
               extractText=0, fformname=None)
        (x,y,w,h) = self.getPageSize(0)
        c = reportlab.pdfgen.canvas.Canvas(fileName,pagesize=(w-x,h-y))
        restoreFormsInMemory(pickledData, c)
        for pageNo in pageNumbers:
            c.doForm('page%d' % pageNo)
            c.showPage()

            # check the rotation and try to preserve it
            #rot = self.getPageRotation(pageNo)
            #if rot:
            #    c._doc.Pages[-1].Rotate = rot
        c.save()


    def rewritePage(self, pageNo, canvas, width, height):
        "Override this to decorate your pages. Draws OVER your graphics"
        pass
        
        #example - uncomment if you wish!
#        #draw over the top.  This text should be near bottom left in red.
#        c = canvas
#        from reportlab.lib.colors import red
#        c.setFillColor(red)
#        c.setFont('Helvetica', 12)
#        c.drawString(0, 0, 'x')
#        c.drawString(10, 10, 'page size (corrected): %d x %d' % (width, height))
#        #overlay a red rectangle 5 points in, just to see if we got the sizes right
#        c.setStrokeColor(red)
#        c.rect(5, 5, width - 10, height - 10)

    def rewriteUnderPage(self, pageNo, canvas, width, height):
        "Override this to decorate your pages. Draws UNDER your graphics"
        pass
        

    def rewrite(self, outFileName):
        """Rewrite PDF, optionally with user decoration
        
        This will create a new PDF file from the existing one.
        It attempts to take care of rotated and cropped input files,
        and always outputs a file with no page-rotation and width the
        width and height you would normally expect.
        
        To decorate a page (e.g. overprint a timestamp), subclass
        PdfExplorer, and implement the rewritePage method:

            def rewritePage(self, pageNo, canvas, width, height):
                #your code here

        Take care to use the passed-in width and height, which will
        have been corrected by rotation and crop box.
        """


        
        pageNumbers = list(range(self.pageCount))
        (names, pickledData) = storeFormsInMemory(self.rawContent,
               pagenumbers=pageNumbers, prefix="page", BBoxes=0,
               extractText=0, fformname=None)
        c = reportlab.pdfgen.canvas.Canvas(outFileName)
        restoreFormsInMemory(pickledData, c)
        for pageNo in pageNumbers:
            (x,y,w,h) = self.getPageSize(0)
            rot = self.getPageRotation(pageNo)
            if rot in [90, 270]:
                w, h = h, w

                #go dumpster diving in the PDF and try to correct for
                #the bounds, which can otherwise clip off the content.
                #Ideally PageCatcher itself would do this when
                #reading in a rotated/cropped document, but I cannot
                #get that to work yet.
                formName = xObjectName(names[pageNo])
                form = c._doc.idToObject[formName]
                form.uppery, form.upperx = form.upperx, form.uppery


    
            #if a crop box is set, the user originally 'saw'
            #a window onto the page specified by an extra box in the
            #PDF with (x1, y1, x2, y2) coords.  We need to shift
            #our underlying form across
            try:
                cropBox = self.getCropBox(pageNo)
            except KeyError:
                cropBox = None

            if cropBox:
                if rot in [90, 270]:
                    cropY1, cropX1, cropY2, cropX2 = cropBox
                else:
                    cropX1, cropY1, cropX2, cropY2 = cropBox
                h = cropY2 - cropY1
                w = cropX2 - cropX1
            c.setPageSize((w,h))

            #user hook - subclass this to overprint
            c.saveState()
            self.rewriteUnderPage(pageNo, c, w, h)
            c.restoreState()

        
            c.saveState()
            if cropBox:
                c.translate(-cropX1, -cropY1)
            c.doForm('page%d' % pageNo)
            c.restoreState()
            
            #user hook - subclass this to overprint
            self.rewritePage(pageNo, c, w, h)
            
            #save it
            c.showPage()

        c.save()








##        import time
##        started = time.clock()
##        print 'entered savePagesAsPdf at %0.2f' % (time.clock() - started)
##        c = reportlab.pdfgen.canvas.Canvas(fileName)
##        print '    canvas created at %0.2f' % (time.clock() - started)
##
##        masterDict = {}
##        for num in pageNumbers:
##            nameToObj = self.getForm(num)
##            masterDict.update(nameToObj)
##        pickledData = cPickle.dumps(masterDict, 1)
##        restoreFormsInMemory(pickledData, c)
####        (names, pickledData) = storeFormsInMemory(self.rawContent,
####               pagenumbers=pageNumbers, prefix="page", BBoxes=0,
####               extractText=0, fformname=None)
##        print '    storeFormsInMemory done at %0.2f' % (time.clock() - started)
##        # pickledData is a cPickled binary dictionary
##        # mapping names to objects
####        nameToObj = {}
####        for number in pageNumbers:
####
##        restoreFormsInMemory(pickledData, c)
##        print '    restored forms at %0.2f' % (time.clock() - started)
##        for pageNo in pageNumbers:
##            c.doForm('page%d' % pageNo)
##            c.showPage()
##
##            # check the rotation and try to preserve it
##            rot = self.getPageRotation(pageNo)
##            if rot:
##                c._doc.Pages[-1].Rotate = rot
##        print '    drew pages at %0.2f' % (time.clock() - started)
##
##        c.save()
##        print '    saved  at %0.2f' % (time.clock() - started)
####        print 'bombing out now...'
####        import sys
####        sys.exit()


    def _extractAnnotations(self):
        """
        Returns list of annotation dictionaries on page.dict

        Here is what an annotation dictionary looks like:
        { 'F': 4,
          'FT': '/Tx',
          'Rect': [108, 577, 407, 594],
          'Subtype': '/Widget',
          'T': 'AgentName',
          'TU': 'blah again',
          'Type': '/Annot'}"""
        self._annotations = {}
        p = PDFParseContext(self.rawContent)
        p.parse()
        (ind, catinfo) = p.catalog
        c = p.compilation
        c.sanitizePages(save=("Type", "Contents",
                              "MediaBox", "ArtBox", "BleedBox", "CropBox", "TrimBox",
                              "Resources", "Rotate", "Annots"))
        #c.sanitizePages()  # this drops info we need
        c.findAllReferences()
        #catalog = \
        c.getReference(catinfo)
        c.doTranslations(catinfo)  # this takes the time!
        c.populatePageList()
        pageCount = len(c.pageList)

        for pageNo in range(pageCount):
            pageId = c.pageList[pageNo]
            page = c.objects[pageId]
            pageDict = page.dict
            if "Annots" in pageDict:
                rawPageAnnots = c.resolve(pageDict["Annots"])
                friendlyPageAnnots = pythonize(rawPageAnnots, c)

            else:
                friendlyPageAnnots = []
            self._annotations[pageNo] = friendlyPageAnnots

    def getAnnotations(self, pageNo):
        if self._annotations is None:
            self._extractAnnotations()
        return self._annotations[pageNo]

    def getAnnotationsTable(self, pageNo):
        """Tries to tabulate into something suitable for a grid report.

        Columns:  fieldtype, name,  rect, text"""
        report = []
        annots = self.getAnnotations(pageNo)
        for annot in annots:
            typ = annot.get('FT',None)
            try:
                typ = FieldTypeExpander[typ[1:]]
            except KeyError:
                pass
            name = annot.get('T',None)
            rect = annot.get('Rect',None)
            text = annot.get('TU',None)
            report.append((typ, name, rect, text))
        return report

def test():
    import sys
    try:
        fn = sys.argv[1]
    except:
        fn = 'PH_Offshore_Q3_2002.pdf'
    exp = PdfExplorer(fn)
    page1ops = exp.getPdfOps(0)
    print('\nextracting...\n')
    from . import pdparse
    xtr = pdparse.PdfTextExtractor().extractText(page1ops)
    print(xtr)

if __name__=='__main__':
    test()



__code__='Gas1]D/U(X&>$Qs4,unKR$<mPb*A;/=6^Yi6#BV%7E/;h/oH>n<2S41EjJ>tSF;KOFlJq.V]bY*s%*/ZJ&Zp(^Z_T@>^XIT\'mk/3hOSq`RINK<&\'TI)denr6p,<l%%O6Lr;#g[1#.@#+N!)ZP8&1&I+!;"X)CD*\\=9GW,/c[d0(BL"UB)n7u56*eRGRgq]GTO\'uGQDb$n3b!LJ0+f@%!W*J\'>RBE0*>M]r!`\'EZ(-/L1=sG$0>h_HD8E.TZHa=Y$Y0^,9FrlWKT5.14=>:kn8h?I6oeM)@HU*\'2M1`+c)O%3<0fsL=X0q3bj\'F)?tAiS@E28t]EWpI$asZR`rN0q+T[?"0"s),"!#`C)2*%QqOg9bA!U84!tGc2+&oCpRs?8E&3j#$nh].IbZfS03dB)&puk;i//G[jp9dr?68i&t;AI3_@hi#kUTHl$KfRGe8&f0=(T7NaoP[5gEiPE)oj-aPZgqB\\;aGW9eXbnXb.<e],XNkA.#^"E_D7,Rd1gHF\'"PF/UaZm44>6EKddr?nkD^.F7(cjs]U\'9r3^-D8MCVimbT\\"oR_d?eQGgjl0HB4jp1\'L;9"Y)"^$\\\')g0nUjFqb-9c6V,C,*G4Gak!!>Nn3Ihh<t:A^8&iqaPV)kAF.s%3H0&;^ZS*#?\\_3h-d%b(1fT,C,jE/"5YO^7(\'V8EX^7aO,h3Y*l2JBOY5D)g>Ka4b;rF6H)/#h1`)`&"oWd_NRpVXXI"qJ=-rmdKYOmu[qp^srj\'1u[UI,`=3`3_RT1Oe>f*8"V/%>!&VK)X]]l3>rrK]C7k3WhHeQ>HZq[k.(s0\\2C&WZ\\JKSG\\0BDq?VHaY2eSt!3@*?"3unTNGKg33@=]BX&%5MXHgh<dAJQeunR"77[)N$NGj06slb8p:$%4NlJH),+#.&+!q;&sY%pUL:f<oh\'`ERH^%hV\\c(T+C5)QaN.D.A;(on5VuoE<1+TI1sPb6_a)[&AjIr]W#\\`kkrVtWW-bB%XtZPLVU,sU`NMTuPXq?7Z/BnV\'G4<$Cb]g)$agseL"IuqSmn+Q63!jKU@H.+^4,/4lmj\\c>c9<578Z;2Hc]A*FlP^<nRT0~>'



#Autogenerated by ReportLab guiedit do not edit
from d_verticalbarchart import VerticalBarChart
from reportlab.graphics.shapes import _DrawingEditorMixin
from rlextra.graphics.guiedit.datacharts import ODBCDataSource, DataAssociation, DataAwareDrawing, Array

class VerticalBarChartDrawing(_DrawingEditorMixin,DataAwareDrawing):
	def __init__(self,width=400,height=200,*args,**kw):
		DataAwareDrawing.__init__(self,width,height,*args,**kw)
		self._add(self,VerticalBarChart(),name='VerticalBarChart',validate=None,desc='The main chart')
		self.height             = 120
		self.width              = 170
		self.dataSource         = ODBCDataSource()
		self.dataSource.sql     = 'SELECT chartId, name, value1, value2 FROM generic_bar'
		self.dataSource.groupingColumn = 0
		self.dataSource.associations   = Array(3, DataAssociation)
		self.dataSource.associations.element00 = DataAssociation(column=0, target='chartId', assocType='scalar')
		self.dataSource.associations.element01 = DataAssociation(column=[2,3], target='VerticalBarChart.data', assocType='tmatrix')
		self.dataSource.associations.element02 = DataAssociation(column=1, target='VerticalBarChart.categoryAxis.categoryNames', assocType='vector')
		self.formats         = ['eps', 'pdf']
		self.verbose         = 1
		self.outDir          = './output/'
		self.fileNamePattern = 'verticalbarchart%03d'
		self.VerticalBarChart.x               = 0
		self.VerticalBarChart.y               = 0


if __name__=="__main__": #NORUNTESTS
	VerticalBarChartDrawing().go()



#!/usr/bin/env python
"""Removes all markup from a string

Should ideally not affect any entities - just remove tags.
Currently (6/9/2012) this aspect seems broken.


>>> 2+2
4
>>> stripTags('hello')
'hello'
>>> stripTags('hello<a href="crap"> world</a>')
'hello world'
>>> stripTags('goodbye<!--stuff--> comments')
'goodbye comments'
>>> stripTags('Entities &amp; stuff')
'Entities & stuff'
>>> stripTags('Fr&#233;gate Island')    #accents etc preserved
'Fr&#233;gate Island'
>>> from reportlab.lib.utils import asBytes
>>> asBytes(stripTags('Space&nbsp;here'))==u'Space\xa0here'.encode('utf8')
True
>>> resolveEntities(b'Fr&#233;gate Island')==b'Fr\\xc3\\xa9gate Island'
True
>>> stripTags('Ch\\xc2\\xa3teau & <tag>Summerhouse</tag>')=='Ch\\xc2\\xa3teau & Summerhouse'
True
>>> stripTags(b'Ch\\xc2\\xa3teau & <tag>Summerhouse</tag>'.decode('utf8'))==b'Ch\\xc2\\xa3teau & Summerhouse'.decode('utf8')
True
"""
from reportlab.platypus.paraparser import HTMLParser, known_entities
from reportlab.lib.utils import uniChr, isUnicode
from rlextra.radxml.xmlutils import nakedAmpFix

def stripTags(text,enc='utf8'):
    return TagStripper().strip(text,enc=enc)

def resolveEntities(text):
    "Turn them to UTF8 text"
    return EntityResolver().resolve(text)

class TagStripper(HTMLParser):
    def strip(self, text, enc='utf8'):
        self._output = []
        self.reset()
        if not isUnicode(text):
            text = text.decode(enc)
        else:
            enc = None
        self.feed(nakedAmpFix(text).replace(u'<br/>',u'<br />'))
        v = u''.join(self._output)
        return v.encode(enc) if enc else v

    def handle_data(self, data):
        self._output.append(data)

    def handle_charref(self, ref):
        "We want char refs to be preserved"
        #print 'ref=',ref, type(ref)
        self.handle_data(u'&#%s;' % ref)

    def handle_entityref(self, name):
        "Handles a named entity.  "
        try:
            v = known_entities[name]
        except:
            v = u'&%s;' % name
        self.handle_data(v)

class EntityResolver(HTMLParser):
    def resolve(self, text, enc='utf8'):
        self._output = []
        self.reset()
        if not isUnicode(text):
            text = text.decode(enc)
        else:
            enc = None
        self.feed(nakedAmpFix(text).replace(u'<br/>',u'<br />'))
        v = u''.join(self._output)
        return v.encode(enc) if enc else v

    def handle_data(self, data):
        self._output.append(data)

    def handle_charref(self, ref):
        try:
            if ref.startswith(u'x'):
                v = int(ref[1:],16)
            else:
                v = int(ref)
            v = uniChr(v)
        except:
            v = u'&#%s;' % ref
        self.handle_data(v)

    def handle_entityref(self, name):
        "Handles a named entity.  "
        try:
            v = known_entities[name]
        except:
            v = u'&%s;' % name
        self.handle_data(v)

if __name__=='__main__':
    import doctest, tagstripper
    doctest.testmod(tagstripper)



__code__='Gas1]>>Na6\'V9$N(D+-6]P/9!@nl<1&Yd8^F[8C#XMj40MP_hn!@lp6bfmrP^shl>J\'^^c;IBA=5\']oVKbZ8qm?c:iF2%+5aZ8sFo_lefI0m(c!>ogt>J_f;c3Vl[RR9+&&;R]$F,fG.?j0SA-!"U[Clhq46gQiQAg;,VNNr",,@C+^A<D8.po$3-;rC)>\\\\SXoSW\\WE9sS>H("`sRECQH2]@e=XUd[)Nl1ZR#pQCpIi#iF7BcmfCB@P<HKs60%YZ^_nDN)tkWu/<r7F2]/8PXCs:<88%Aa"/8dG3;-)5Wm;M4O+`Nn0m<ap)1ISmHK9Uq?JP\\\\>MZph%e!ZdLI-Sl-a_af\\K36kCfja:-2.SW$%+hNYX4[N0cDU($<O/J7Q^\'&KBDHet?B>*6\\*FI+I/\'nim@g5\\O4-c_X@,Zoc=3\'tf1#bKPu[$/G&FjMPKDWE[clXkdPgPf8c7!m!t%pE:a^*<h*1`,S4!<bM2%G;?92#g<PbkF5VKrTFQk%\\bH@0Y0PbV8O3,WRXo!@#65j./BK4f+<SQ-.M%OY-.Cd+c#-\\7qG3]l`Il/PcOiAB)LIm3E6.j2]:3mi880rXZ$6&?S@n\'`[Q7p,rB>EVE\'OLKmBo6cH;Z37\'0p]$m!RQg#$TfkMKU&$!A!H14>/(@[ba9DEk*jL-j,4$>A/s+Qm:`sZraddbN5j[aA\'H!u"`B]@_l/aCltKmV87Rd$cca8PrZg?kG\'3o2rihLSGDFQ2<$d+d@GJrm<\\YdYP>VrJ6S[mZmLc&EhUo9ucn7t9\'&!7YeE!H=(L**3\\QC:Le]YYmtnLW-)#<A:J`;d4kB>4i>nmuV>F@U?=3MS6R4#LL2eaCgAH^a[PG;>C@3KP>B+PW-pd%/W)(7;QiWp^11AVcR8mie>and`5M<bt%1;ar%\\Y&/Csb"^\'lfjLSY7!_6""+CS:V\'Q;-QmUu/gi-aYCJ")S%r;:rB?=:9jLc5V\\/f(EP[Jr?&g*ehc?SCACq>~>'



from __future__ import unicode_literals
import os, sys
from rlextra.radxml.xmlmap import LazyAttrs, MapController, MapNode, SpecialTransform
from rlextra.radxml.xmlutils import TupleTreeWalker
from reportlab.lib.utils import asUnicode
import pyRXPU
import collections

def xhtmlDocFromXhtmlFragment(xhtmlFrag,enc='utf8'):
    r = []
    #r.append('<?xml version="1.0" encoding="UTF-8"?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">')
    r.append('<html><head><title></title></head><body>')
    xhtmlFrag=xhtmlFrag.replace('<p><table>','<table>').replace('</table></p>','</table>')
    r.append(asUnicode(xhtmlFrag,enc=enc))
    r.append('</body></html>')
    r = ''.join(r)
    return r

def curry(func, *args, **kwargs):
    class C:
        def __init__(self, func, args, kw):
            self.func = func
            self.args = args
            self.kw = kw
        def __call__(self, *args, **kw):
            nkw = self.kw
            nkw.update(kw)
            return self.func(*(self.args + args), **nkw)
    return C(func, args, kwargs)

def pTransform(c, paraStyle='normal', tagAttrs={}):
    "take image out of p and put p in imageAndFlowables"
    attrs = extractTagAttrs('p',tagAttrs,style=paraStyle)
    if '<imageAndFlowables' not in c:
        return '<para%s>%s</para>' % (attrs, c)
    f = []
    #b = []
    s = '%s' % c
    while '<imageAndFlowables' in s:
        im = s[s.index('<imageAndFlowables'):s.index('</imageAndFlowables>') + 20]
        s = s.replace(im, '')
        im = im.split('><')
        f.append(im[0] + '>')
        #b.append('<' + im[1])
    f[-1] = f[-1] + '<para%s>%s</para>' % (attrs, s)
    f = [a + '</imageAndFlowables>' for a in f]
    f = ''.join(f) #+ ''.join(b)
    return f

def aTransform(sdict, name):
    href = sdict.get('href', '')
    if href.startswith('http://') or href.startswith('https://'):
        return '<a href="%s">%s</a>' % (href, sdict['__content__'])
    else:
        return sdict['__content__']

def aTransformMailto(sdict, name):
    """Transform of <a> which allows mailto links."""
    href = sdict.get('href', '')
    if href.startswith('http://') or href.startswith('https://')\
        or href.startswith('mailto:'):
        return '<a href="%s">%s</a>' % (href, sdict['__content__'])
    else:
        return sdict['__content__']

class ImageTransform(SpecialTransform):
    """Transforms XHTML 'img' tag to 'imageAndFlowables', but only if it's there.
    
    If not, we transform to '' so that missing images on the server don't cause
    RML to choke when creating the guide.
    """
    amap=dict(src='imageName',
              width='imageWidth',
              height='imageHeight',
              )

    def __init__(self, pathTransform=None,allowMissing=False, imageWidth=None, imageHeight=None):
        self.pathTransform = pathTransform
        self.allowMissing = allowMissing
        self.imageWidth = imageWidth
        self.imageHeight = imageHeight

    def __call__(self, sdict, name):
        D = sdict[name]._D
        src = D['src']
        if self.pathTransform:
            src = D['src'] = self.pathTransform(src)
        allowMissing = self.allowMissing
        if allowMissing:
            if isinstance(allowMissing, collections.Callable) and not allowMissing(src):
                return ''
        elif not os.path.isfile(src) and not src.startswith('rml:'):
            #image not found, so exclude it from the RML.  
            return ''

        ND = {}
        amap=self.amap
        K=list(amap.keys())
        for k,v in D.items():
            if k in K:
                ND[amap[k]] = v
        for a in ('imageWidth','imageHeight'):
            v = getattr(self,a,None)
            if v:
                ND[a] = v
        return '<imageAndFlowables %s imageSide="left"></imageAndFlowables>' % str(LazyAttrs(ND))

def extractTagAttrs(tag, tagAttrs, **existingAttrs):
    d = tagAttrs.get(tag,{})
    d.update(existingAttrs)
    return ''.join(' %s="%s"' % (k,v) for (k,v) in d.items() if v is not None)

def weakUpdateTagAttrs(tag,tagAttrs,**existingAttrs):
    d = tagAttrs.setdefault(tag,{})
    for k in existingAttrs:
        if k not in d:
            d[k] = existingAttrs[k]

def xhtml2rml(xml, paraStyle='normal', tableStyle='noPaddingStyle', bulletStyle='bullet',
                pathTransform=None, imageTransformKwds={}, allowMailtoLinks=False,
                useModernLists=True, ulStyle=None, olStyle=None, liParaStyle=None,
                tagAttrs={},
                ):
    """Convert chunk of our mini-html to RML.
    
    >>> xhtml2rml('text')=='text'               #avoid spurious whitespace
    True
    >>> xhtml2rml('2m * 2m = 4m<sup>2</sup>')=='2m * 2m = 4m<sup>2</sup>'   #why it matters - space appears!
    True
    >>> xhtml2rml('<p>test</p>')=='<para style="normal">test</para>'
    True
    >>> xhtml2rml('<h1>test</h1>',tagAttrs=dict(h1=dict(style='myh1style')))=='<para style="myh1style">test</para>'
    True
    >>> xhtml2rml('<h6>test</h6>',tagAttrs=dict(h6=dict(style='myh6style')))=='<para style="myh6style">test</para>'
    True
    >>> xhtml2rml('<p>test</p>', paraStyle='custom')=='<para style="custom">test</para>'
    True
    >>> from rlextra.radxml.html_cleaner import cleanBlocks
    >>> xhtml2rml(cleanBlocks('<p>aaaa <img src="rml:img1">bbbb<p>cccc<img src="rml:img2"> dddd</p> eeeee</p>'))=='<imageAndFlowables  imageName="rml:img1" imageSide="left"><para style="normal">aaaa bbbb</para></imageAndFlowables><imageAndFlowables  imageName="rml:img2" imageSide="left"><para style="normal">cccc dddd</para></imageAndFlowables><para style="normal"> eeeee</para>'
    True
    >>> xhtml2rml(cleanBlocks('<ol><li>one</li><li>two</li></ol>'), olStyle='xxx')=='<ol style="xxx"><li><para style="normal">one</para></li><li><para style="normal">two</para></li></ol>'
    True
    >>> xhtml2rml(cleanBlocks('<ul><li>one</li><li>two</li></ul>'), ulStyle='xxx')=='<ul style="xxx"><li><para style="normal">one</para></li><li><para style="normal">two</para></li></ul>'
    True
    >>> xhtml2rml(cleanBlocks('<ul><li>one</li><li>two</li></ul>'), ulStyle='xxx', tagAttrs=dict(li=dict(bulletOffsetY=2)))=='<ul style="xxx"><li bulletOffsetY="2"><para style="normal">one</para></li><li bulletOffsetY="2"><para style="normal">two</para></li></ul>'
    True
    """
    xml = xhtmlDocFromXhtmlFragment(xml)
    M = MapController()
    tagAttrs = tagAttrs.copy()
    
    M[""] = '%(__content__)s'
    M["html"] = '%(__content__)s'
    M["body"] = '%(__content__)s'
    M["head"] = '%(__content__)s'
    M["title"] = '%(__content__)s'
    M["p"] = '%(__content__)s'
    M["p"].transformContent(curry(pTransform, paraStyle=paraStyle))
    M["img"] = MapNode(None, '%(__attrs__)s')
    if pathTransform: imageTransformKwds['pathTransform'] = pathTransform
    M["img"].addTransform('__attrs__', ImageTransform(**imageTransformKwds))
    M["table"] = '<blockTable%s>%%(__content__)s</blockTable>' % extractTagAttrs('table',tagAttrs,style=tableStyle)
    M["tr"] = '<tr>%(__content__)s</tr>'
    paraAttrs = extractTagAttrs('para',tagAttrs,style=paraStyle)
    attrs = (extractTagAttrs('td',tagAttrs),paraAttrs)
    M["td"] = '<td%s><para%s>%%(__content__)s</para></td>' % attrs
    M["th"] = '<td%s><para%s>%%(__content__)s</para></td>' % attrs
    M["b"] = '<b>%(__content__)s</b>'
    M["i"] = '<i>%(__content__)s</i>'
    M["u"] = '<u>%(__content__)s</u>'
    M["sup"] = '<sup>%(__content__)s</sup>'
    M["sub"] = '<sub>%(__content__)s</sub>'
    M["strong"] = MapNode('<b>%(__content__)s</b>','')
    M["em"] = '<i>%(__content__)s</i>'
    M["br"] = MapNode(None, '<br/>')
    for n in '123456':
        tag = 'h'+n
        weakUpdateTagAttrs(tag,tagAttrs,style=tag)
        M[tag] = '<para%s>%%(__content__)s</para>' % extractTagAttrs(tag,tagAttrs)
    if useModernLists:
        liParaStyle = liParaStyle or paraStyle
        M["ul"] = '<ul%s>%%(__content__)s</ul>' % extractTagAttrs('ul',tagAttrs,style=ulStyle)
        M["ol"] = '<ol%s>%%(__content__)s</ol>' % extractTagAttrs('ol',tagAttrs,style=olStyle)
        M["li"] = '<li%s><para%s>%%(__content__)s</para></li>' % (extractTagAttrs('li',tagAttrs),extractTagAttrs('para',tagAttrs,style=liParaStyle))
    else:
        #oldstyle for now
        M["ul"] = '%(__content__)s'
        M["ol"] = '%(__content__)s'
        M["li"] = '<para%s><bullet>&bull;</bullet>%%(__content__)s</para>' % extractTagAttrs('bulletPara',tagAttrs,style=bulletStyle)
    M["address"] = '<para%s>%%(__content__)s</para>' % paraAttrs
    M["span"] = '%(__content__)s'
    M["a"] = MapNode('%(__attrs__)s', None)
    if allowMailtoLinks:
        M["a"].addTransform('__attrs__', aTransformMailto)
    else:
        M["a"].addTransform('__attrs__', aTransform)
    parser = pyRXPU.Parser(ExpandCharacterEntities=0, ExpandGeneralEntities=0)
    xml = parser.parse(xml)
    return M.process(xml, isTupleTree=True)

class ImageFinder(TupleTreeWalker):
    def __init__(self, tree):
        TupleTreeWalker.__init__(self, tree)
        self.images = []
    def startElement(self, tagName, attrs):
        if tagName == 'img':
            if 'src' in attrs:
                path = attrs['src']
                self.images.append(path)

def findImages(xml):
    "Returns lists of all images referred to in markup"
    xml = asUnicode(xml)
    validDoc = xhtmlDocFromXhtmlFragment(xml)  #adds 'html','head etc. to our fragment
    parser = pyRXPU.Parser(ReturnUTF8=1, ExpandCharacterEntities=0, ExpandGeneralEntities=0)
    tree = parser.parse(validDoc)
    walker = ImageFinder(tree)
    walker.go()
    return walker.images

# Testing

stylesheet = '''<initialize>
<alias id="style.normal" value="style.Normal"/>
</initialize>
<paraStyle name="h1" fontName="Courier-Bold" fontSize="15" spaceBefore = "0.5 cm" />
<paraStyle name="h2" fontName="Courier-Bold" fontSize="14" spaceBefore = "0.5 cm" />
<paraStyle name="h3" fontName="Courier-Bold" fontSize="13" spaceBefore = "0.5 cm" />
<paraStyle name="h4" fontName="Courier-Bold" fontSize="12" spaceBefore = "0.5 cm" />
<paraStyle name="normal" fontName="Helvetica" fontSize="10" leading="12"/>
<paraStyle name="bullet" parent="normal" leftIndent = "0.5cm"/>
<blockTableStyle id="noPaddingStyle" spaceAfter="0.5 cm">
    <blockAlignment value="left"/>
    <blockFont name="Helvetica-Oblique"/>
    <lineStyle kind="GRID" colorName="black"/>
    <lineStyle kind="OUTLINE" colorName="black" thickness="2"/>
    <blockBackground colorName="pink" start="0,0" stop="-1,0"/>
    <blockBackground colorName="yellow" start="0,0" stop="-1,0"/>
    <blockBackground colorName="yellow" start="0,0" stop="-1,0"/>
    <listStyle name="list_normal" bulletFontSize="10" bulletFontName="Helvetica"/>
</blockTableStyle>'''

def indent(str):
    return '\n'.join(['\t%s' % l for l in str.split('\n')])

def testfile(path):
    from rlextra.radxml.html_cleaner import cleanBlocks
    from rlextra.rml2pdf.rml2pdf import invalid_rml
    import time
    import traceback

    f = open(path, 'r')
    try:
        html = cleanBlocks(f.read(), aHrefTr='http://reportlab.com')
        open(path+'.cleaned', 'w').write(html)
        print('Created ' + path+'.cleaned')
        b = float(len(html))/1024.0
        print('Testing %s (%d kB)' % (path, int(b)))
        try:
            start = time.time()
            rml = xhtml2rml(html)
            d = time.time() - start
        except:
            print(indent(traceback.format_exc()))
            print()
            sys.exit()
        else:
            open(path+'.rml', 'w').write(rml)
            print('Generated %s | Speed: %f kB/s' % (path+'.rml', b/d))
            errs = invalid_rml(rml, paragraph=False, stylesheet=stylesheet, saveAs=path+'.pdf')
            if not errs:
                print('Created ' + path+'.pdf')
            else:
                print('Error:', errs)
                sys.exit()
    finally:
        f.close()

def testdir(path):
    'Recursively looks for .html or .xml files in path and tries to convert them to rml and pdf.'
    for d in os.listdir(path):
        p = os.path.join(path, d)
        if os.path.isdir(p):
            testdir(p)
        elif os.path.isfile(p) and (p.endswith('.html') or p.endswith('.xml')):
            testfile(p)

if __name__=='__main__':
    # Runs doc tests and if a list of paths are passed, recursively converts .html and .xml files in there
    import doctest
    doctest.testmod()
    
    for path in sys.argv[1:]:
        testdir(path)
